// ########################################################################
// ## generator.cc                                                       ##
// ## Zufllige Erzeugung von Graphen                                    ##
// ## Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de            ##
// ##           WWW Homepage:     http://ibm.rhrz.uni-bonn.de/~uzs6pq    ##
// ########################################################################

#define RandCubicName "RandCubic 1.0"

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <string.h>

#include "../tdgf/tdgf.h"
#include "../zufall/zufall.h"

#include "anzahltabelle.h"
#include "anzahltabelle.icc"
#include "randcubic.h"


// Hauptprogramm
int32 main(card32 argc, char** argv)
{
   card32 knoten   = 0;
   card32 kstart   = 0;
   card32 kende    = 0;
   card32 anzahl   = 1;
   bool bitMatrix  = true;
   char  titel[128];
   char* copyright = NULL;
   char* programm  = RandCubicName;
   char* name      = NULL;
   bool  ausgabe   = false;
   bool  gtitel    = false;
   bool  typA      = false;
   bool  typB      = false;
   bool  typC      = false;
   bool  typD      = false;
   ofstream          os;
   card32            i,n,typ,neg;

   if(argc == 1) {
      cerr << "Aufruf: " << argv[0] << " {Parameter}" << endl;
      cerr << "-a[#]    - Minimale Knotenzahl setzen" << endl;
      cerr << "-b[#]    - Maximale Knotenzahl setzen" << endl;
      cerr << "-k[#]    - Knotenzahl setzen (entspricht min == max)" << endl;
      cerr << "-#[#]    - Anzahl der zu erzeugenden Graphen" << endl;
      cerr << "-f[Name] - Dateiname fr Ausgabe (TDGF)" << endl;
      cerr << "-t[Text] - Titel setzen" << endl;
      cerr << "-c[Text] - Copyright setzen" << endl;
      cerr << "-$       - Ausgabe zustzlich auf Bildschirm" << endl;
      cerr << "-A       - A-Graphen erzeugen" << endl;
      cerr << "-B       - B-Graphen erzeugen" << endl;
      cerr << "-C       - C-Graphen erzeugen" << endl;
      cerr << "-D       - D-Graphen erzeugen" << endl;
      cerr << "-w       - Graphentyp als Untertitel angeben" << endl;
      cerr << "-y       - Adjazenzmatrix im Byte-Format statt Bit-Format speichern" << endl;
      cerr << "-z       - Anzahltabelle zeigen" << endl;
      exit(0);
   }

   // Parameter auswerten
   titel[0] = 0x00;
   for(i = 1;i < argc;i++) {
      if(!(strncmp(argv[i],"-t",2))) strcat(titel,&argv[i][2]);
      else if(!(strncmp(argv[i],"-c",2))) copyright = &argv[i][2];
      else if(!(strncmp(argv[i],"-a",2))) kstart    = atoi(&argv[i][2]);
      else if(!(strncmp(argv[i],"-b",2))) kende     = atoi(&argv[i][2]);
      else if(!(strncmp(argv[i],"-k",2))) knoten    = atoi(&argv[i][2]);
      else if(!(strncmp(argv[i],"-#",2))) anzahl    = atoi(&argv[i][2]);
      else if(!(strncmp(argv[i],"-f",2))) name      = &argv[i][2];
      else if(!(strcmp(argv[i],"-A"))) typA    = true;      
      else if(!(strcmp(argv[i],"-B"))) typB    = true;      
      else if(!(strcmp(argv[i],"-C"))) typC    = true;      
      else if(!(strcmp(argv[i],"-D"))) typD    = true;      
      else if(!(strcmp(argv[i],"-w"))) gtitel  = true;  
      else if(!(strcmp(argv[i],"-$"))) ausgabe = true;  
      else if(!(strcmp(argv[i],"-z"))) cout << tabelle << endl;  
      else if(!(strcmp(argv[i],"-y"))) bitMatrix = false;
   } 
   if(name == NULL) ausgabe = true;
   if(anzahl == 0) anzahl = 1;
   if(knoten == 0) {
      if(kstart < 4) kstart = 4;
      if(kstart > tabelle.Eintraege()) kstart = tabelle.Eintraege();
      if(kende < kstart) kende = kstart;
      if(kende > tabelle.Eintraege()) kende = tabelle.Eintraege();
   }
   else {
      if(knoten < 4) knoten = 4;
      if(knoten > tabelle.Eintraege()) knoten = tabelle.Eintraege();
      kstart = knoten;
      kende  = knoten;
   }
   if((typA == false) && (typB == false) && (typC == false)) typD = true;
   i = 0;
   if(typA) i++;
   if(typB) i++;
   if(typC) i++;
   if(typD) i++;
   if(i > 1) gtitel = true;
   if(kstart == kende) {
      if(((kstart % 2) == 0) && (typA)) {
         cerr << "Es sind nur gerade Knotenzahlen eingestellt!" << endl;
         cerr << "Keine Generierung von A-Graphen mglich!" << endl;
         exit(1);
      }
      if(((kstart % 2) == 1) && (typB || typC || typD)) {
         cerr << "Es sind nur ungerade Knotenzahlen eingestellt!" << endl;
         cerr << "Keine Generierung von B/C/D-Graphen mglich!" << endl;
         exit(1);
      }     
   }
   if(titel[0] == 0x00) {
      if((typA == false) && (typB == false) && (typC == false)) {
         if(kstart == kende)
            sprintf(titel,"Zufllig erzeugte kubische Graphen mit %d Knoten",kstart);
         else
            sprintf(titel,"Zufllig erzeugte kubische Graphen mit %d bis %d Knoten",kstart,kende);
      }
      else {
         if(kstart == kende)
            sprintf(titel,"Zufllig erzeugte Graphen mit %d Knoten",kstart);
         else
            sprintf(titel,"Zufllig erzeugte Graphen mit %d bis %d Knoten",kstart,kende);
      }
   }
   if(name) {
      if(!ErstelleTDGF(os,name,titel,copyright,programm)) {
         cerr << "Kann Datei \"" << name << "\" nicht erstellen!" << endl;
         exit(1);
      }
   }


   // Zufallszahlengenerator initialisieren
   Randomize();


   // Informationen ausgeben
   cout << "Parameter fr die Generierung:" << endl;
   cout << "Knotenzahlen von " << kstart << " bis " << kende << endl;
   cout << "Typen:    "
        << (typA ? " A-Graph" : "") 
        << (typB ? " B-Graph" : "")
        << (typC ? " C-Graph" : "")
        << (typD ? " D-Graph" : "") << endl;
   cout << "Anzahl:    " << anzahl << endl;
   cout << "Titel:     " << titel << endl;
   cout << "Copyright: " << copyright << endl << endl;


   // Graphen erzeugen und ausgeben
   i = 0;
   while(i < anzahl) {
      typ = Random(4);
      if(kstart == kende) 
         n = kstart;
      else
         n = Random(1 + kende - kstart) + kstart - 1;
    
      RandCubic graph(n);
      graph.InitialisiereLabels(n);
      switch(typ) {
         case A_GRAPH:
           if((typA == true) && ((n % 2) == 1) ) {
              graph.GraphA(n);
              if(ausgabe) cout << "A-Graph: " << endl << graph << endl;
              if(name) graph.Schreibe(os,bitMatrix,gtitel ? typ : 0);
              i++;            
           }           
          break;
         case B_GRAPH:
           if((typB == true) && ((n % 2) == 0) ) {
              graph.GraphB(n,neg);
              if(ausgabe) cout << "B-Graph: " << endl << graph << endl;
              if(name) graph.Schreibe(os,bitMatrix,gtitel ? typ : 0,neg);
              i++;            
           }           
          break;
         case C_GRAPH:
           if((typC == true) && ((n % 2) == 0) ) {
              graph.GraphC(n,neg);
              if(ausgabe) cout << "C-Graph: " << endl << graph << endl;
              if(name) graph.Schreibe(os,bitMatrix,gtitel ? typ : 0,neg);
              i++;            
           }           
          break;
         case D_GRAPH:
           if((typD == true) && ((n % 2) == 0) ) {
              graph.GraphD(n);
              if(ausgabe) cout << "D-Graph: " << endl << graph << endl;
              if(name) graph.Schreibe(os,bitMatrix,gtitel ? typ : 0);
              i++;            
           }           
          break;
      }              
   }
   return 0;
}
