// ########################################################################
// ## graph.h:                                                           ##
// ## Klasse zur Verwaltung von Graphen                                  ##
// ## Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de            ##
// ##           WWW Homepage:     http://ibm.rhrz.uni-bonn.de/~uzs6pq    ##
// ########################################################################

#ifndef GRAPH_H
#define GRAPH_H

#include <iostream.h>

#include "../system.h"


// Speicherformat fr die Adjazenzmatrix
#define BYTE_MATRIX 0
#define BIT_MATRIX  1


// Klasse fr Graphen
class Graph
{
   // ===== Konstruktoren und Destruktor ==================================
   public:
   Graph();
   Graph(const card32 n, const card16 flags = BYTE_MATRIX);
   Graph(const Graph& original);
   ~Graph();


   // ===== Zugriff auf Daten =============================================
   inline card32 Kanten() const;
   inline card32 Knoten() const;
   inline char*  Titel() const;
   bool          Bipart(const card32 knoten) const;
   inline card8* Biparts() const;
   bool          Label(const card32 knoten,
                      char* speicher) const;
   inline char*  Labels() const;
   
   inline Graph* Nachfolger() const;
   inline void   SetzeNachfolger(Graph* t);
   void          SetzeTitel(const char* neuerTitel);
   void          SetzeBipart(const card32 knoten, const bool menge);      
   void          SetzeBiparts(const card8* neueBiparts);      
   void          SetzeLabel(const card32 knoten,
                            const char* neueLabels);
   void          SetzeLabels(const char* neueLabels);


   // ===== Operatoren ====================================================
   void            operator= (const Graph& original);
   friend ostream& operator<<(ostream& os, const Graph& graph);

    
   // ===== Verwaltung der Adjazenzmatrix =================================
   void Neu();
   inline void   Verbinde  (const card32 a, const card32 b);
   inline void   Trenne    (const card32 a, const card32 b);
   inline bool   Verbunden (const card32 a, const card32 b) const;
   inline bool   Getrennt  (const card32 a, const card32 b) const;
   inline bool   Verbindbar(const card32 a, const card32 b) const;
   inline card32 Grad(const card32 n) const;


   // ===== Laden und Speichern ===========================================
   bool Lese(istream& is);
   bool Schreibe(ostream& os);

   
   // ===== Interne Variablen =============================================
   private:
   card16 flags;
   card16 knoten;
   card16 kanten;
   card8* adjazenzMatrix;
   char*  titel;
   char*  labels;
   card8* biparts;
   Graph* nachfolger;       

    
   // ===== Interne Funktionen ============================================
   private:
   void Initialisiere(const card32 n, const card16 flags);
   void GibFrei();
   void Clone(const Graph& original);    
   void SetzeKante(const card32 a, const card32 b,
                   const bool neueKante);
};


// Berechnung des Speicherbedarfs der Adjazenzmatrix
inline card32 AdjazenzBytes(const card32 n, const card16 flags);

#endif
