// ########################################################################
// ## viewerwidget.cc                                                    ##
// ## Viewer fr Graphenlisten                                           ##
// ## Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de            ##
// ##           WWW Homepage:     http://ibm.rhrz.uni-bonn.de/~uzs6pq    ##
// ########################################################################

const char* KGraphViewerName    = "KGraphViewer";
const char* KGraphViewerVersion = "1.0";


#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <string.h>

#include <qapp.h>
#include <qfont.h>
#include <qpainter.h>
#include <qprinter.h>
#include <qfiledlg.h>
#include <qmenubar.h>
#include <qpopmenu.h>
#include <qkeycode.h>
#include <qmsgbox.h>
#include <qprogdlg.h>

#include <kapp.h>
#include <ktopwidget.h>
#include <ktoolbar.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <kprogress.h>
#include <kurl.h>

#include "../system.h"
#include "../tdgf/tdgf.h"
#include "../tdgf/graph.h"
#include "../tdgf/graph.icc"

#include "graphwidget.h"
#include "graphwidget.icc"
#include "viewerwidget.h"
#include "viewerwidget.icc"


QList<ViewerWidget> ViewerWidget::windowList;


// Konstruktor fr ViewerWidget
ViewerWidget::ViewerWidget()
 : KTopLevelWidget()
{
    setMinimumSize(300,150);

    // Datei-Men
    file = new QPopupMenu();
    file->insertItem(i18n("&New Window..."),     idFileNewWindow);
    file->insertItem(i18n("&Close Window"),      idFileCloseWindow);
    file->insertSeparator();
    file->insertItem(i18n("&Open Graph List..."),idFileOpenList);
    file->insertItem(i18n("&About this List..."),idFileAboutList);
    file->insertItem(i18n("&Print this List..."),idFilePrintList);
    file->insertSeparator();
    file->insertItem(i18n("&Quit"),              idFileQuit);

    // Go-Men
    goTo = new QPopupMenu();
    goTo->insertItem(i18n("Go to Ne&xt Page"),    idGoToNext);
    goTo->insertItem(i18n("Go to Pr&evious Page"),idGoToPrev);
    goTo->insertSeparator();
    goTo->insertItem(i18n("Go to &First Page"),idGoToFirst);
    goTo->insertItem(i18n("Go to &Last Page"), idGoToLast);

    // Optionen-Men
    options = new QPopupMenu();
    options->insertItem(i18n("Show &Tool Bar"),  idOptionsToolBar);
    options->insertItem(i18n("Show Stat&us Bar"),idOptionsStatusBar);
    options->insertSeparator();
    options->insertItem(i18n("Add &Row"),      idOptionsYPlus);
    options->insertItem(i18n("D&elete Row"),   idOptionsYMinus);
    options->insertItem(i18n("Add &Column"),   idOptionsXPlus);
    options->insertItem(i18n("De&lete Column"),idOptionsXMinus);

    // Hilfe-Men
    QPopupMenu* help = kapp->getHelpMenu(true, 
       QString(KGraphViewerName) + " - " + QString(KGraphViewerVersion) +
       "\n\nCopyright Thomas Dreibholz\n"
       "E-Mail:   uzs6pq@ibm.rhrz.uni-bonn.de\n"
       "Homepage: http://ibm.rhrz.uni-bonn.de/~uzs6pq");

    // Item-Einstellungen setzen
    options->setItemChecked(idOptionsToolBar,zeigeToolBar);
    options->setItemChecked(idOptionsStatusBar,zeigeStatusBar);
    file->setItemEnabled(idFilePrintList,false);
    file->setItemEnabled(idFileAboutList,false);
    goTo->setItemEnabled(idGoToPrev, false);
    goTo->setItemEnabled(idGoToNext, false);
    goTo->setItemEnabled(idGoToFirst,false);
    goTo->setItemEnabled(idGoToLast, false);

    // Menleiste
    menuBar = new KMenuBar(this);
    CHECK_PTR(menuBar);
    menuBar->insertItem(i18n("&File"),file);
    menuBar->insertItem(i18n("&Goto"),goTo);
    menuBar->insertItem(i18n("&Options"),options);
    menuBar->insertSeparator();
    menuBar->insertItem(i18n("&Help"),help);
    menuBar->show();
    setMenu(menuBar);
    connect(goTo,    SIGNAL(activated(int)), SLOT(menuCallback(int)));
    connect(options, SIGNAL(activated(int)), SLOT(menuCallback(int)));
    connect(file,    SIGNAL(activated(int)), SLOT(menuCallback(int)));
    connect(help,    SIGNAL(activated(int)), SLOT(menuCallback(int)));


    // Tool-Bar
    KIconLoader *lader = kapp->getIconLoader();
    toolBar = new KToolBar(this);
    toolBar->insertButton(lader->loadIcon("exit.xpm"),
                          idFileCloseWindow, true,
                          i18n("Close"));
    toolBar->insertButton(lader->loadIcon("filenew.xpm"),
                          idFileNewWindow, true,
                          i18n("New Window"));
    toolBar->insertButton(lader->loadIcon("fileopen.xpm"),
                          idFileOpenList, true,
                          i18n("Open a file"));
    toolBar->insertSeparator();
    toolBar->insertButton(lader->loadIcon("back.xpm"),
                          idGoToPrev, false,
                          i18n("Go to previous page"));
    toolBar->insertButton(lader->loadIcon("forward.xpm"),
                          idGoToNext, false,
                          i18n("Go to next page"));
    toolBar->insertButton(lader->loadIcon("start.xpm"),
                          idGoToFirst, false,
                          i18n("Go to first page"));
    toolBar->insertButton(lader->loadIcon("finish.xpm"),
                          idGoToLast,  false,
                          i18n("Go to last page"));
    toolBar->insertSeparator();
    toolBar->insertButton(lader->loadIcon("help.xpm"),
                          idFileAboutList, false, i18n("About this List"));
    toolBar->insertButton(lader->loadIcon("fileprint.xpm"),
                          idFilePrintList, false, i18n("Print this List"));
    toolBar->insertSeparator();
    toolBar->insertButton(lader->loadIcon("deleterow.xpm"),
                          idOptionsYMinus, false, i18n("Delete Row"));
    toolBar->insertButton(lader->loadIcon("addrow.xpm"),
                          idOptionsYPlus,  false, i18n("Add Row"));
    toolBar->insertSeparator();
    toolBar->insertButton(lader->loadIcon("deletecolumn.xpm"),
                          idOptionsXMinus, false, i18n("Delete Column"));
    toolBar->insertButton(lader->loadIcon("addcolumn.xpm"),
                          idOptionsXPlus,  false, i18n("Add Column"));

    addToolBar(toolBar);
    toolBar->setBarPos(KToolBar::Top);
    toolBar->show();
    connect(toolBar,SIGNAL(clicked(int)), SLOT(menuCallback(int)));    

    // Status-Bar
    statusBar = new KStatusBar(this);
    statusBar->insertItem(i18n("Page: ------ of ------"),   idStatus);
    statusBar->insertItem(i18n("Welcome to KGraphViewer!"), idTitle);
    statusBar->setInsertOrder(KStatusBar::RightToLeft);
    setStatusBar(statusBar);

    // Graph-Display
    display = new GraphWidget(this);
    setView(display);
    connect(display,SIGNAL(neueSeite()), SLOT(statusBarUpdate()));

    // Drag and Drop Zone
    dropZone = new KDNDDropZone(this , DndURL);
    connect(dropZone,SIGNAL(dropAction(KDNDDropZone *)),
                     SLOT(dropEvent(KDNDDropZone *)));

    // ViewerWidget anzeigen
    zeigeToolBar   = true;
    zeigeStatusBar = true;
    show(); 
    windowList.append(this);
}


// Desstruktor fr ViewerWidget
ViewerWidget::~ViewerWidget()
{
}


// Widget schlieen
void ViewerWidget::closeEvent(QCloseEvent* event)
{
   windowList.remove(this);
   delete toolBar;
   delete menuBar;
   delete statusBar;
   delete display;
   delete options;
   delete file;
   delete goTo;
   delete dropZone;
   if(windowList.isEmpty()) {
      kapp->quit();
   }
   event->accept();
}


// Men- und Toolbarverwaltung
void ViewerWidget::menuCallback(int id)
{
   switch(id) {
      case idGoToNext:
          display->NaechsteSeite();
       break;
      case idGoToPrev:
          display->VorherigeSeite();
       break;
      case idGoToFirst:
          display->ErsteSeite();
       break;
      case idGoToLast:
          display->LetzteSeite();
       break;
      case idOptionsXMinus:
          display->Einstellungen(-1,0);
       break;
      case idOptionsXPlus:
          display->Einstellungen(1,0);
       break;
      case idOptionsYMinus:
          display->Einstellungen(0,-1);
       break;
      case idOptionsYPlus:
          display->Einstellungen(0,1);
       break;
      case idFileNewWindow:
         new ViewerWidget;  
       break;
      case idFileOpenList:
         Oeffnen(NULL);
        break;
      case idFileCloseWindow:
         close();
       break;
      case idFilePrintList:
         Drucken();
       break;
      case idFileAboutList:
         Information();
       break;
      case idOptionsToolBar:
         zeigeToolBar = !zeigeToolBar;
         enableToolBar(zeigeToolBar ? KToolBar::Show : KToolBar::Hide);
         options->setItemChecked(id,zeigeToolBar);
       break;
      case idOptionsStatusBar:
         zeigeStatusBar = !zeigeStatusBar;
         enableStatusBar(zeigeStatusBar ? KStatusBar::Show : KStatusBar::Hide);
         options->setItemChecked(id,zeigeStatusBar);
       break;
      case idFileQuit:
       {
          for(ViewerWidget *w = windowList.first();w; w = windowList.next())
             w->close();
          kapp->exit();
       }  
       break;
   }
}


// Statusbar updaten
void ViewerWidget::statusBarUpdate()
{
   char str[60];
   sprintf(str,i18n("Page: %d of %d"), display->Seite(),display->Seiten());
   statusBar->changeItem(str,idStatus);
}


// Graphenliste oeffnen
bool ViewerWidget::Oeffnen(const char* fileName)
{
   QString name;

   if(fileName == NULL) {
      name = KFileDialog::getOpenFileName(NULL,"*.tdgf");
      if(name.isEmpty()) return(false);
   } 
   else {
      name = QString(fileName);
   }

   QList<Graph> liste;
   ifstream     is;
   TDGF_Header  tdgf;

   switch(OeffneTDGF(is,name.data(),&tdgf,&info)) { 
      case dos:
         QMessageBox::information(NULL,KGraphViewerName,
                                  i18n("Unable to read file!"),
                                  i18n("&Abort"),0);
         return(false);
       break;
      case format:
         QMessageBox::information(NULL,KGraphViewerName,
            i18n("This is not a TDGF format graph file!"),
            i18n("&Abort"),0);
         return(false);
       break;
      case kompression:
         QMessageBox::information(NULL,KGraphViewerName,
            i18n("TDGF file compression is not implemented!"),
            i18n("&Abort"),0);
         return(false);
       break;
      case byteOrder:
         QMessageBox::information(NULL,KGraphViewerName,
            i18n("This TDGF file has an incompatible byte order!"),
            i18n("&Abort"),0);
         return(false);
       break;
      default:
       break;       
   }  

   // Graphenliste laden
   statusBar->changeItem(info.Titel,idTitle);
   display->Neu();
   Graph graph;
   for(;;) {
      if(!graph.Lese(is)) break;
      display->FuegeAn(graph);
   }
   display->Reset();

   // Funktionen anwhlbar/gesperrt setzen
   bool ok = (display->Anzahl() > 0);
   file->setItemEnabled(idFilePrintList,ok);
   file->setItemEnabled(idFileAboutList,ok);
   goTo->setItemEnabled(idGoToPrev,  ok);
   goTo->setItemEnabled(idGoToNext,  ok);
   goTo->setItemEnabled(idGoToFirst, ok);
   goTo->setItemEnabled(idGoToLast,  ok);
   toolBar->setItemEnabled(idFilePrintList,ok);
   toolBar->setItemEnabled(idFileAboutList,ok);
   toolBar->setItemEnabled(idGoToPrev,  ok);
   toolBar->setItemEnabled(idGoToNext,  ok);
   toolBar->setItemEnabled(idGoToFirst, ok);
   toolBar->setItemEnabled(idGoToLast,  ok);
   toolBar->setItemEnabled(idOptionsXPlus,  ok);
   toolBar->setItemEnabled(idOptionsYPlus,  ok);
   toolBar->setItemEnabled(idOptionsXMinus, ok);
   toolBar->setItemEnabled(idOptionsYMinus, ok);
   return(true);
}


// Laden mittels Drag and Drop
void ViewerWidget::dropEvent(KDNDDropZone* dropZone)
{
    QStrList& list = dropZone->getURLList();
    char *s;

    for(s = list.first();s != 0L;s = list.next()) {
       KURL url(s);
       if(s == list.getFirst()) {
          Oeffnen(url.path());
       }
       else {
          ViewerWidget* neuerViewer = new ViewerWidget;
          if(neuerViewer->Oeffnen(url.path()) == false) {
             delete neuerViewer;
          }
       }
    }
}


// Graphenliste drucken
void ViewerWidget::Drucken()
{
   if(display->Anzahl() < 1) return;

   QPrinter printer;
   int32 seite  = 1;
   int32 seiten = (card32)ceil((double)display->Anzahl() / (double)(display->AnzahlX() * display->AnzahlY()));

   printer.setOrientation(QPrinter::Landscape);
   printer.setCreator("KGraphViewer - Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de");
   printer.setDocName("KGraphViewer PostScript Output File");
   printer.setOutputFileName(i18n("output.ps"));
   printer.setFromTo(1,seiten);
   printer.setMinMax(1,seiten);
   
   printer.setOutputToFile(true);
   if(printer.setup(this)) {
      if(display->Anzahl() > 0) {
         QPainter painter;
         QProgressDialog progress(i18n("Printing! Please wait..."),i18n("Abort Printing!"),seiten,this,NULL,true);
         progress.setCaption(i18n("Printing Progress"));         
         progress.setProgress(0);
         progress.show();
         painter.begin(&printer);
         painter.setWindow(-15,-30,891+60,629+60);

         int32 start = (printer.fromPage() - 1) * display->AnzahlX() * display->AnzahlY();
         int32 ende  = printer.toPage() * display->AnzahlX() * display->AnzahlY();

         for(int32 i = start;i < ende;i += display->AnzahlX() * display->AnzahlY()) {
            progress.setProgress(seite);
            if(progress.wasCancelled()) break;
            if(i >= display->Anzahl()) break;

            // berschrift drucken
            if(i > start) {
               printer.newPage();
               seite++;
            }

            char str[128];
            painter.setPen(black);
            painter.setFont(QFont("times",16,QFont::Bold));
            painter.drawText(20,0,850,25,AlignLeft | AlignVCenter,info.Titel);
            sprintf(str,i18n("Page %d of %d"),seite,seiten);
            painter.drawText(20,0,850,25,AlignRight | AlignVCenter,str);
            painter.setFont(QFont("times",12,QFont::Bold));
            if(info.Copyright[0] != 0x00) {
               sprintf(str,"%s %s",i18n("Copyright"),info.Copyright);
               painter.drawText(20,604,850,25,AlignRight | AlignVCenter,str);
            }
            if(info.Programm[0] != 0x00) {
               sprintf(str,"%s %s %s %02d.%02d.%04d, %d:%02d:%02d %s",
                       i18n("Generated with"),info.Programm,i18n("on"),
                       info.Tag,info.Monat,info.Jahr,
                       info.Stunde,info.Minute,info.Sekunde,info.Zone);                           
               painter.drawText(20,604,850,25,AlignLeft | AlignVCenter,str);
            }

            // Graphen drucken
            display->ZeichneSeite(i,&painter,0,25,890,579);
         }
         painter.end();
      }
   } 
}

// Informationen ber Graphen-Datei
void ViewerWidget::Information()
{
   char str[768];

   sprintf(str,"%s\n\nCopyright: %s\n%s %s %s %02d.%02d.%4d, %d:%02d:%02d %s\n\n%s: %d",
               info.Titel,info.Copyright,
               i18n("Generated with"),info.Programm,i18n("on"),
               info.Tag,info.Monat,info.Jahr,
               info.Stunde,info.Minute,info.Sekunde,info.Zone,
               i18n("Total Graph Count"),
               display->Anzahl());

   QMessageBox::information(NULL,KGraphViewerName,str,i18n("&Okay"),0); 
}

#include "viewerwidget.moc"
